unit bmp_u;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, jpeg, StdCtrls,Menus,clipbrd,

  service_u;

type
  //class 24 bit-bitmap
  tbmp=class(TBitmap)
  public
    ba:pbytearray; //scanline-pointer
    ba_y:integer;  //scanline-row

    constructor create;override;
    procedure clr;
    procedure size_set(w,h:integer);
    procedure fill_col(col:tcolor);
    procedure fill_rgbcol(col:trgbcol);
    function ba_set(y:integer):bool;

    function col_get(x:integer):tcolor;
    function rgbcol_get(x:integer):trgbcol;
    function col_set(x:integer;col:tcolor):bool;
    function rgbcol_set(x:integer;rgbcol:trgbcol):bool;
    function rec_no_rgbcol_count(rgbcol:trgbcol;var rec:trect):int64;
  end;

implementation

uses main_u;

//--------------------------------------------------------------------
constructor tbmp.create;
begin
  inherited;
  pixelformat:=pf24bit;
  ba_y:=-1;
  ba:=nil;
end;

procedure tbmp.size_set(w,h:integer);
begin
  if w<>width then Width:=w;
  if h<>height then height:=h;
end;

procedure tbmp.clr;begin size_set(10,10);end;


procedure tbmp.fill_col(col:tcolor);
begin
  canvas.pen.width:=1;
  canvas.pen.color:=col;
  canvas.pen.style:=pssolid;
  canvas.Brush.color:=col;
  canvas.brush.style:=bssolid;
  rectangle(canvas.handle,0,0,width,height);
end;

procedure tbmp.fill_rgbcol(col:trgbcol);
begin
  fill_col(rgb(col.r,col.g,col.b));
end;

function tbmp.ba_set(y:integer):bool;
begin
  result:=true;
  //if y is null get scanline every time
  if(y<>0)and(y=ba_y) then exit;
  ba:=scanline[y];
  //set scanline-row-parameter
  ba_y:=y;
end;

//get pixel-color in trgb-format
function tbmp.rgbcol_get(x:integer):trgbcol;
begin
  x:=x*3;
  result.r:=ba[x+2];
  result.g:=ba[x+1];
  result.b:=ba[x+0];
end;

//get pixel-color in tcolor-format
function tbmp.col_get(x:integer):tcolor;
begin
  result:=service_u.rgbcol2col(rgbcol_get(x));
end;

function tbmp.rgbcol_set(x:integer;rgbcol:trgbcol):bool;
begin
  x:=x*3;
  ba[x+2]:=rgbcol.r;
  ba[x+1]:=rgbcol.g;
  ba[x+0]:=rgbcol.b;
  result:=true;
end;

function tbmp.col_set(x:integer;col:tcolor):bool;
begin
  result:=rgbcol_set(x,service_u.col2rgbcol(col));
end;

//find a rect in the bitmap with no appearance of the color rgbcol
function tbmp.rec_no_rgbcol_count(rgbcol:trgbcol;var rec:trect):int64;
var
  ba:pbytearray;
  x,y:integer;
begin
  result:=0;

  //fill rec width dummy-params
  rec:=rect(-1,-1,-1,-1);

  //check all rows of bitmap
  for y:=0 to height-1 do begin
    ba:=scanline[y];

    //check all pixels of ba
    for x:=0 to width-1 do begin

      //pixel-color is rgbcol? Then ignore it
      if(ba[x*3+2]=rgbcol.r)and(ba[x*3+1]=rgbcol.g)and(ba[x*3]=rgbcol.b)then continue;

      //no, it's another color
      inc(result);

      //calculate new rectangle-values
      if (x<rec.left)or(rec.left=-1)then rec.left:=x;
      if (y<rec.top)or(rec.top=-1)then rec.top:=y;

      if (x>rec.right)or(rec.right=-1)then rec.right:=x;
      if (y>rec.bottom)or(rec.bottom=-1)then rec.bottom:=y;
    end;
  end;

  //color rgbcol not found?
  if result=0 then exit;

  //adjust right and bottom of rec
  rec.right:=rec.right+1;
  rec.bottom:=rec.bottom+1;
end;

end.

