unit selection_u;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, Menus, XPMan,jpeg,math,

  service_u,bmp_u;

type
  tselection=class(tobject)
    range_ok:bool;
    rec:trect;
    style:tselection_style;
    bmp:tbmp;
    org_bmp:tbmp;

    border_outer_col:tcolor;
    border_inner_col:tcolor;

    mouse_click_pt:tpoint;
    mouse_hit_left_ok:bool;
    mouse_hit_top_ok:bool;

    merge_style:tmerge_style;
    transparency:byte;

    to_sprite_ok:bool;

    constructor create;
    destructor destroy;override;

    procedure MouseDown(Button:TMouseButton;Shift:TShiftState;X,Y:Integer);
    procedure MouseMove(Shift:TShiftState;X,Y:Integer);
    procedure MouseUp(Button:TMouseButton;Shift:TShiftState;X,Y:Integer);

    procedure new_start(x,y:integer);
    procedure new_change(x,y:integer);
    procedure new_end;

    procedure border;
    procedure clear;

    procedure sizing(x,y:integer);
    procedure moving(x,y:integer);

    procedure rec_set(new_rec:trect);

    function hit(x,y:integer):bool;
    function cursor_get(x,y:integer):tcursor;
    function rec_width:integer;
    function rec_height:integer;

    function rec_norm(arec:trect):trect;

    function merge_rgbcol(merged_rgbcol:trgbcol;x,y:integer;var hit_ok:bool):trgbcol;

    //procedure to_sprite;
    procedure to_sprite_start;
    procedure to_sprite_paint;
    procedure to_sprite_end;

    procedure dblclick;

    function is_sprite:bool;
  end;

implementation

uses main_u,sprite_u,sprite_list_u;

//--------------------------------------------------------------------------
constructor tselection.create;
begin
  inherited;
  org_bmp:=tbmp.create;
  bmp:=tbmp.create;
  merge_style:=ms_always;
  style:=ss_rectangle;
  to_sprite_ok:=false;
  border_outer_col:=clblack;
  border_inner_col:=clwhite;
end;

destructor tselection.destroy;
begin
  org_bmp.Free;
  bmp.Free;
  inherited;
end;

function tselection.hit(x,y:integer):bool;
begin
  result:=
    (x>=rec.left)and(x<rec.right)and
    (y>=rec.top)and(y<rec.bottom);
end;

procedure tselection.MouseDown(Button:TMouseButton;Shift:TShiftState;X,Y:Integer);
begin
  if not range_ok then begin
    //neue range beginnen
    new_start(x,y);
    exit;
  end;

  //range nicht getroffen: mit false raus
  if not hit(x,y) then exit;

  main_f.Cursor_set(cursor_get(x,y));
  mouse_click_pt:=point(x-rec.left,y-rec.top);
end;

//--------------------------------------------------------------------------
procedure tselection.MouseMove(Shift:TShiftState;X,Y:Integer);
begin
  if not main_f.mouse_down_ok then begin
    main_f.cursor_set(cursor_get(x,y));
    exit;
  end;

  if not range_ok then begin
    //neue range erweitern
    new_change(x,y);
    exit;
  end;

  //range existiert bereits

  //range nicht getroffen: mit false raus
  if(main_f.cursor=crdefault)and not hit(x,y) then begin
    exit;
  end;

  if      main_f.cursor=crsizeall  then moving(x,y)
  else if main_f.cursor<>crdefault then sizing(x,y);

  //dont paint with sprites
  if is_sprite then exit;

  //direkt durchpausen?
  if not main_f.selection_to_sprite_chb.checked then begin
    if not(ssctrl in shift) then exit;
  end;

  //neustart vom durchmalen?
  if not to_sprite_ok then to_sprite_start;

  to_sprite_paint;
end;

//--------------------------------------------------------------------------
procedure tselection.MouseUp(Button:TMouseButton;Shift:TShiftState;X,Y:Integer);
begin
  if is_sprite then exit;

  if button=mbright then begin
    //lsche range
    clear;
    exit;
  end;

  if not range_ok then begin
    //neue range abschliessen
    new_end;
    exit;
  end;

  if to_sprite_ok then to_sprite_end;
end;

procedure tselection.clear;
begin
  rec:=rect(0,0,0,0);
  range_ok:=false;
//  main_f.pb_update(true);
end;

procedure tselection.new_start(x,y:integer);
begin
  clear;
  if      main_f.selection_style_rectangle_rb.checked then style:=ss_rectangle
  else if main_f.selection_style_ellipse_rb.checked   then style:=ss_ellipse
                                                      else style:=ss_freestyle;
  //start rectangle
  rec:=rect(x,y,x+1,y+1);

  //not freestyle? nothing more to do
  if style<>ss_freestyle then exit;

  main_f.freestyle_c:=0;
  main_f.freestyle_a[main_f.freestyle_c].x:=x;
  main_f.freestyle_a[main_f.freestyle_c].y:=y;
  rec_set(rec);
  //main_f.pb_update(true);
end;

procedure tselection.new_change(x,y:integer);
var
  new_rec:trect;
begin

  if style=ss_freestyle then begin

    //index for new point
    inc(main_f.freestyle_c);

    //put point in array
    main_f.freestyle_a[main_f.freestyle_c].x:=x;
    main_f.freestyle_a[main_f.freestyle_c].y:=y;

    //calculate new selection-rectangle
    new_rec.left:=min(rec.left,x);
    new_rec.top:=min(rec.top,y);
    new_rec.right:=max(rec.right,x);
    new_rec.bottom:=max(rec.bottom,y);
  end
  else begin
    new_rec:=rec;
    new_rec.right:=x;
    new_rec.bottom:=y;
  end;

  rec_set(new_rec);
  main_f.pb_update(true);
end;

procedure tselection.new_end;
var
  src:tbmp;
  mask_bmp:tbmp;
  x,y:integer;
  rgbcol:trgbcol;
  r:integer;
begin
  try

    //grsse der ziel-bitmap
    bmp.size_set(rec_width,rec_height);

    if main_f.selection_src_org_rb.checked then
      src:=main_f.pic_bmp
    else begin
      main_f.pb_update(false);
      src:=main_f.preview_bmp;
    end;

    //src.SaveToFile('d:\tmp\1.bmp');

    //rectangle einfach kopieren
    if style=ss_rectangle then begin
      bitblt(
        bmp.canvas.Handle,0,0,bmp.width,bmp.Height,
        src.Canvas.Handle,rec.left,rec.top,
        srccopy
      );
      exit;
    end;

    mask_bmp:=tbmp.create;
    try
      mask_bmp.size_set(rec_width,rec_height);
      mask_bmp.fill_rgbcol(main_f.transp_rgbcol);
      mask_bmp.canvas.pen.color:=clblack;
      mask_bmp.canvas.brush.color:=clblack;
      mask_bmp.canvas.brush.style:=bssolid;

      if style=ss_ellipse then begin
        mask_bmp.canvas.ellipse(rect(0,0,rec_width,rec_height));
      end
      else begin
        for r:=0 to main_f.freestyle_c do begin
          main_f.freestyle_a[r].x:=main_f.freestyle_a[r].x-rec.left;
          main_f.freestyle_a[r].y:=main_f.freestyle_a[r].y-rec.top;
        end;
        mask_bmp.canvas.Polygon(slice(main_f.freestyle_a,main_f.freestyle_c));
      end;
      //mask_bmp.SaveToFile('d:\tmp\2.bmp');

      for y:=0 to mask_bmp.height-1 do begin
        src.ba_set(y+rec.top);
        mask_bmp.ba_set(y);
        bmp.ba_set(y);
        for x:=0 to mask_bmp.width-1 do begin
          if mask_bmp.col_get(x)=clblack then rgbcol:=src.rgbcol_get(x+rec.left)
                                         else rgbcol:=main_f.transp_rgbcol;
          bmp.rgbcol_set(x,rgbcol);
        end;
      end;
      //bmp.SaveToFile('d:\tmp\3.bmp');

    finally
      mask_bmp.Free;
    end;

  finally
    main_f.freestyle_c:=0;
    merge_style:=tmerge_style(main_f.selection_merge_style_cb.ItemIndex);
    transparency:=main_f.selection_transparency_sb.Position;
    range_ok:=true;
    org_bmp.assign(bmp);
    rec_set(rec);
    main_f.pb_update(true);
  end;
end;

function tselection.cursor_get(x,y:integer):tcursor;
var
  q,dw,dh:integer;
  l,t,r,b:integer;
  d:double;
begin
  result:=crdefault;
  if not hit(x,y)then exit;

  q:=10;
  l:=rec.left;
  t:=rec.top;
  r:=rec.right;
  b:=rec.bottom;

  //die hlften des crop-rechtecks
  d:=l+((r-l)-q)/2;dw:=round(d);
  d:=t+((b-t)-q)/2;dh:=round(d);


  //obere linke seite?
  mouse_hit_left_ok:=(x<l+q);
  mouse_hit_top_ok:=(y<t+q);

  //okay, ein sizepoint erwischt?
  if     (x<l+q)and(y<t+q)             then result:=crsizenwse //oben links
  else if(x>dw-q)and(x<dw+q)and(y<t+q) then result:=crsizens   //oben mitte
  else if(x>=r-q)and(y<t+q)            then result:=crsizeNESW //oben rechts

  else if(x>=r-q)and(y>dh-q)and(y<dh+q)then result:=crsizewe   //mitte rechts
  else if(x>=r-q)and(y>=b-q)           then result:=crsizenwse //unten rechts

  else if(x>dw-q)and(x<dw+q)and(y>=b-q)then result:=crsizens   //unten mitte

  else if(x<l+q)and(y>=b-q)            then result:=crsizenesw //unten links
  else if(x<l+q)and(y>dh-q)and(y<dh+q) then result:=crsizewe   //mitte links

  else                                      result:=crsizeall; //nein

end;

function tselection.is_sprite:bool;
begin
  result:=(border_outer_col=clteal);
end;

function tselection.rec_width:integer;begin  result:=rec.right-rec.left;end;
function tselection.rec_height:integer;begin result:=rec.bottom-rec.top;end;

//---------------------------------------------------------------------
procedure tselection.moving(x,y:integer);
var
  w,h:integer;
  tmp_rec:trect;
begin
  w:=rec_width;
  h:=rec_height;

  tmp_rec.left:=x-mouse_click_pt.x;
  tmp_rec.top:=y-mouse_click_pt.y;

  tmp_rec.right:=tmp_rec.left+w;
  tmp_rec.bottom:=tmp_rec.top+h;

  rec_set(tmp_rec);
  main_f.pb_update(true);
end;

//--------------------------------------------------------------------------
procedure tselection.sizing(x,y:integer);
var
  tmp_rec:trect;
begin
  tmp_rec:=rec;
  if main_f.cursor=crsizewe then begin
    if mouse_hit_left_ok then tmp_rec.left:=x  //linke seite erwischt
                         else tmp_rec.right:=x; //rechte seite erwischt
  end
  else if main_f.cursor=crsizens then begin
    if mouse_hit_top_ok then tmp_rec.top:=y
                        else tmp_rec.bottom:=y;
  end
  else begin
    if mouse_hit_left_ok then tmp_rec.left:=x
                         else tmp_rec.right:=x;

    if mouse_hit_top_ok then tmp_rec.top:=y
                        else tmp_rec.bottom:=y;
  end;

  rec_set(tmp_rec);
  main_f.pb_update(true);
end;

function tselection.rec_norm(arec:trect):trect;
var
  rec:trect;
begin
  rec.left:=min(arec.left,arec.Right);
  rec.top:=min(arec.top,arec.bottom);
  rec.right:=max(arec.left,arec.Right);
  rec.bottom:=max(arec.top,arec.bottom);
  if (rec.Right-rec.left)=0 then rec.Right:=rec.Right+1;
  if (rec.bottom-rec.Top)=0 then rec.bottom:=rec.bottom+1;
  result:=rec;
end;

//----------------------------------------------------------------------------
procedure tselection.rec_set(new_rec:trect);
begin
  rec:=rec_norm(new_rec);

  if not range_ok then exit;

  if(rec_width=bmp.width)and(rec_height=bmp.height)then exit;

  bmp.size_set(rec_width,rec_height);
  SetStretchBltMode(bmp.canvas.handle,coloroncolor);
  stretchblt(
    bmp.Canvas.Handle,0,0,bmp.width,bmp.height,
    org_bmp.canvas.Handle,0,0,org_bmp.width,org_bmp.height,
    srccopy
  );
end;

//----------------------------------------------------------------------------
procedure tselection.to_sprite_start;
begin
  main_f.sprite_bmp.size_set(main_f.preview_bmp.width,main_f.preview_bmp.height);
  main_f.sprite_bmp.fill_rgbcol(main_f.transp_rgbcol);
  to_sprite_ok:=true;
end;

procedure tselection.to_sprite_paint;
var
  x,y:integer;
  sprite_y,sprite_x:integer;
  rgbcol:trgbcol;
begin
  if style=ss_rectangle then begin
    bitblt(
      main_f.sprite_bmp.Canvas.Handle,rec.left,rec.top,rec_width,rec_height,
      bmp.canvas.Handle,0,0,
      srccopy
    );
    //main_f.sprite_bmp.debug_save(true,'to_sprite_paint',1,'');
    exit;
  end;

  for y:=0 to bmp.height-1 do begin
    sprite_y:=y+rec.top;
    if sprite_y<0 then continue;
    if sprite_y>main_f.sprite_bmp.height-1 then break;

    main_f.sprite_bmp.ba_set(sprite_y);
    bmp.ba_set(y);

    for x:=0 to bmp.width-1 do begin
      sprite_x:=x+rec.left;
      if sprite_x<0 then continue;
      if sprite_x>main_f.sprite_bmp.width-1 then continue;

      rgbcol:=bmp.rgbcol_get(x);

      if main_f.is_transp_rgbcol(rgbcol)then continue;

      main_f.sprite_bmp.rgbcol_set(sprite_x,rgbcol);
    end;
  end;

end;

procedure tselection.to_sprite_end;
var
  sprite:tsprite;
begin
  to_sprite_ok:=false;
  sprite:=tsprite.create;
  sprite.assign(self);
  main_f.sprite_lb.ItemIndex:=sprite_list_u.add(sprite);
  //main_f.pctrl.activepage:=main_f.control_ts;
  main_f.pb_update(true);
end;

procedure tselection.dblclick;
begin
  if is_sprite then exit;
  to_sprite_start;
  to_sprite_paint;
  to_sprite_end;
end;

//------------------------------------------------------------------------
function tselection.merge_rgbcol(merged_rgbcol:trgbcol;x,y:integer;var hit_ok:bool):trgbcol;

  function rgbcol_transform(merged_rgbcol,rgbcol:trgbcol):trgbcol;
  var
    i,ii:byte;
  begin
    result:=merged_rgbcol;

    //ignore transparent pixel
    if main_f.is_transp_rgbcol(rgbcol)then exit;

    //pixel to paint?
    if(merge_style<>ms_always)and(not main_f.is_transp_rgbcol(merged_rgbcol))then begin
      if merge_style=ms_darker then begin
        if not service_u.rgbcol_is_darker(rgbcol,merged_rgbcol) then exit;
      end
      else if merge_style=ms_lighter then begin
        if not service_u.rgbcol_is_lighter(rgbcol,merged_rgbcol) then exit;
      end;
    end;

    if transparency<>0 then begin
      i:=transparency;ii:=100-i;
      rgbcol.r:=trunc((merged_rgbcol.r*i)/100)+round((rgbcol.r*ii)/100);
      rgbcol.g:=trunc((merged_rgbcol.g*i)/100)+round((rgbcol.g*ii)/100);
      rgbcol.b:=trunc((merged_rgbcol.b*i)/100)+round((rgbcol.b*ii)/100);
    end;

    hit_ok:=true;
    result:=rgbcol;
  end;

var
  rgbcol:trgbcol;
begin
  hit_ok:=false;
  result:=merged_rgbcol;

  //exists a selection?
  if not range_ok then exit;

  if to_sprite_ok then begin

    //get to-sprite-painted pixel
    main_f.sprite_bmp.ba_set(y);
    rgbcol:=main_f.sprite_bmp.rgbcol_get(x);

    result:=rgbcol_transform(merged_rgbcol,rgbcol);
    if
      (result.r<>merged_rgbcol.r)or
      (result.g<>merged_rgbcol.g)or
      (result.b<>merged_rgbcol.b)
    then merged_rgbcol:=result;
  end;

  if not hit(x,y) then exit;

  //get pixel
  bmp.ba_set(y-rec.top);
  rgbcol:=bmp.rgbcol_get(x-rec.left);

  result:=rgbcol_transform(merged_rgbcol,rgbcol);
end;

//--------------------------------------------------------------
procedure tselection.border;
var
  cv:tcanvas;
  l,t,w,h:integer;
  hit_quad:byte;
  pt,prev_pt:tpoint;
  r:integer;
begin
  if not range_ok and(style=ss_freestyle)and(main_f.freestyle_c>0)then begin
    cv:=main_f.preview_bmp.canvas;
    cv.brush.style:=bsclear;
    cv.pen.width:=1;
    pt:=main_f.freestyle_a[0];
    cv.MoveTo(pt.x,pt.y);
    prev_pt:=pt;
    for r:=1 to main_f.freestyle_c do begin
      pt:=main_f.freestyle_a[r];
      cv.pen.Color:=border_outer_col;
      cv.lineTo(pt.x,pt.y);
      cv.MoveTo(prev_pt.x+1,prev_pt.y+1);
      cv.pen.Color:=border_inner_col;
      cv.lineTo(pt.x+1,pt.y+1);
      cv.MoveTo(pt.x,pt.y);
      prev_pt:=pt;
    end;
    pt:=main_f.freestyle_a[0];
    cv.lineTo(pt.x,pt.y);
    exit;
  end;


  l:=rec.left;t:=rec.top;w:=rec.right-rec.left;h:=rec.Bottom-rec.top;
  if(w=0)or(h=0)then exit;

  cv:=main_f.preview_bmp.canvas;

  //aussenrahmen
  cv.brush.style:=bsclear;
  cv.Pen.color:=border_outer_col;rectangle(cv.Handle,l,t,l+w,t+h);

  //innenrahmen
  cv.Pen.color:=border_inner_col;rectangle(cv.Handle,l+1,t+1,l+w-1,t+h-1);

  //hit-points
  cv.Pen.color:=border_outer_col;cv.Brush.color:=border_inner_col;cv.Brush.Style:=bssolid;

  hit_quad:=10;

  //hits oben
  cv.Rectangle(l,t,l+hit_quad,t+hit_quad);
  cv.Rectangle(l+(w-hit_quad)div 2,t,l+(w+hit_quad)div 2,t+hit_quad);
  cv.Rectangle(l+w-hit_quad,t,l+w,t+hit_quad);

  //hits links und rechts
  cv.Rectangle(l,t+(h-hit_quad)div 2,l+hit_quad,t+(h+hit_quad)div 2);
  cv.Rectangle(l+w-hit_quad,t+(h-hit_quad)div 2,l+w,t+(h+hit_quad)div 2);

  //hits unten
  cv.Rectangle(l,t+h-hit_quad,l+hit_quad,t+h);
  cv.Rectangle(l+(w-hit_quad)div 2,t+h-hit_quad,l+(w+hit_quad)div 2,t+h);
  cv.Rectangle(l+w-hit_quad,t+h-hit_quad,l+w,t+h);

  if style=ss_ellipse then begin;

    //ellipse aussenrahmen
    cv.brush.style:=bsclear;
    cv.Pen.color:=border_outer_col;ellipse(cv.Handle,l,t,l+w,t+h);

    //ellipse innenrahmen
    cv.Pen.color:=border_inner_col;ellipse(cv.Handle,l+1,t+1,l+w-1,t+h-1);
  end;
end;

end.

