unit service_u;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  clipbrd;

const
  _caption='SPRITE-PAINTER V1.0';
  _background_color=clbtnface;
  _cr=#13#10;

  //definition of transparent color
  _transp_col_r=254;
  _transp_col_g=1;
  _transp_col_b=2;

  //max number of freestyle-points
  _freestyle_max=5000;

  //max number of sprites
  _sprite_max=100;

type
  trgbcol=record
    r,g,b:byte;
  end;
  tselection_style=(ss_rectangle,ss_ellipse,ss_freestyle);
  tmerge_style=(ms_always,ms_lighter,ms_darker);

//some service functions
function col2rgbcol(col:tcolor):trgbcol;
function rgbcol2col(rgbcol:trgbcol):tcolor;
function rgbcol_is_darker(rgbcol1,rgbcol2:trgbcol):bool;
function rgbcol_is_lighter(rgbcol1,rgbcol2:trgbcol):bool;

procedure stream_write(strm:tstream;s:string);
function stream_tag_section_read(strm:tstream;tag:string):string;
function string_tag(section,tag,default:string):string;
function string_tag_int(section,tag:string;default:integer):integer;

procedure stream_clipboard_copy(fmt:Cardinal;strm:TStream);
function stream_clipboard_paste(fmt:Cardinal;strm:TStream):bool;

implementation

//transform tcolor to rgbcol
function col2rgbcol(col:tcolor):trgbcol;
begin
  result.r:=getrvalue(col);
  result.g:=getgvalue(col);
  result.b:=getbvalue(col);
end;

//transform trgbcol to tcolor
function rgbcol2col(rgbcol:trgbcol):tcolor;
begin
  result:=rgb(rgbcol.r,rgbcol.g,rgbcol.b);
end;

//check if rgbcol1 is darker than rgbcol2
function rgbcol_is_darker(rgbcol1,rgbcol2:trgbcol):bool;
begin
  result:=((rgbcol1.r-rgbcol2.r)+(rgbcol1.g-rgbcol2.g)+(rgbcol1.b-rgbcol2.b)<0);
end;

//check if rgbcol1 is lighter than rgbcol2
function rgbcol_is_lighter(rgbcol1,rgbcol2:trgbcol):bool;
begin
  result:=not rgbcol_is_darker(rgbcol1,rgbcol2);
end;

//---------------------------------------------------------------
//read a complete tag-section out of stream
//
//a tag-section is build like in html-pages:
//
//<tag>....</tag>
//
//---------------------------------------------------------------
function stream_tag_section_read(strm:tstream;tag:string):string;
const
  _max=10*1024;
var
  s,ss:string;
  tag_ok:bool;
  ch:char;
  c:integer;
begin
  //search beginning of tag-section
  tag_ok:=false;s:='';ss:='';c:=0;
  repeat
    strm.Read(ch,1);
    s:=s+ch;

    //tag-start found?
    if not tag_ok then if pos('<'+tag+'>',s)>0 then tag_ok:=true;

    //if tag-flag set, save characters of string
    if tag_ok then ss:=ss+ch;
    inc(c);

    //end of tag-section found?
  until (pos('</'+tag+'>',s)>0)or(c>_max);

  //too much characters found?
  if c>_max then begin
    //yep: exit with error
    result:='-ERR';
    exit;
  end;

  //cut off some unnecessary chars
  s:=copy(ss,2,length(ss)-length(tag)-4);
  result:=s;
end;

//get string-value of a special tag out of the string-tag-section "s"
function string_tag(section,tag,default:string):string;
var
  c:integer;
begin
  result:=default;c:=pos('<'+tag+'>',section);if c=0 then exit;
  section:=copy(section,c+length(tag)+2,length(section));
  c:=pos('</'+tag+'>',section);if c=0 then exit;
  result:=copy(section,1,c-1);
end;

//get integer-value of a special tag out of the string-tag-section "s"
function string_tag_int(section,tag:string;default:integer):integer;
begin
  try
    result:=strtoint(string_tag(section,tag,inttostr(default)));
  except
    result:=default;
  end;
end;

//save a string in stream
procedure stream_write(strm:tstream;s:string);
begin
  strm.Writebuffer(s[1],length(s));
end;


//copy a stream into clipboard, tagged as "sprite-painter-pic"
procedure stream_clipboard_copy(fmt:Cardinal;strm:TStream);
var
  hMem:THandle;
  pMem:Pointer;
begin
  strm.Position:=0;

  hMem:=GlobalAlloc(GHND or GMEM_DDESHARE,strm.Size);
  if hMem=0 then begin
    application.MessageBox('Out of memnory','ERROR',mb_ok);
    exit;
  end;

  pMem:=GlobalLock(hMem);
  if pMem=nil then begin
    GlobalFree(hMem);
    application.MessageBox('Out of memnory','ERROR',mb_ok);
    exit;
  end;

  strm.Read(pMem^,strm.Size);
  strm.Position:=0;
  GlobalUnlock(hMem);
  Clipboard.Open;
  try
    Clipboard.SetAsHandle(fmt,hMem);
  finally
    Clipboard.Close;
  end;
end;

//paste a stream from clipboard if tagged as "sprite-painter-pic"
function stream_clipboard_paste(fmt:Cardinal;strm:TStream):bool;
var
  hMem:THandle;
  pMem:Pointer;
begin
  result:=false;
  hMem:=Clipboard.GetAsHandle(fmt);
  if hMem=0 then begin
    application.MessageBox('Clipboard-GetHandle-Error','ERROR',mb_ok);
    exit;
  end;

  pMem:=GlobalLock(hMem);
  if pMem=nil then begin
    application.MessageBox('Could not lock global handle','ERROR',mb_ok);
    exit;
  end;

  strm.Write(pMem^,GlobalSize(hMem));
  strm.Position:=0;
  GlobalUnlock(hMem);
  result:=true;
end;

end.
