unit sprite_list_u;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, Menus, XPMan,jpeg,math,clipbrd,

  service_u,bmp_u,sprite_u;

function add(sprite:tsprite):integer;
function sprite_active:tsprite;
function sprite_get(inx:integer):tsprite;
procedure down;
procedure up;
procedure stream_save(strm:tstream);
procedure stream_load(strm:tstream);
procedure delete(inx:integer);
procedure delete_active;
procedure clear;
function count:integer;
function inx_check(inx:integer):bool;

procedure clipboard_copy;
procedure clipboard_paste;

implementation

uses main_u;

function add(sprite:tsprite):integer;
begin
  inc(main_f.sprite_c);
  sprite.name:='Sprite '+inttostr(main_f.sprite_c);
  main_f.sprite_lb.AddItem(sprite.name,sprite);
  main_f.pic_bmp.Modified:=true;
  result:=count-1;
end;

function sprite_get(inx:integer):tsprite;
begin
  result:=nil;if not inx_check(inx) then exit;
  result:=tsprite(main_f.sprite_lb.Items.Objects[inx]);
end;

function sprite_active:tsprite;
begin
  result:=tsprite(sprite_get(main_f.sprite_lb.ItemIndex));
end;

procedure down;
var
  i:integer;
begin
  i:=main_f.sprite_lb.ItemIndex;
  if(i<0)or(i>count-2) then exit;
  main_f.sprite_lb.Items.Exchange(i,i+1);
  main_f.pic_bmp.Modified:=true;
  main_f.pb_update(true);
end;

procedure up;
var
  i:integer;
begin
  i:=main_f.sprite_lb.ItemIndex;
  if(i<1)or(i>count-1) then exit;
  main_f.sprite_lb.Items.Exchange(i,i-1);
  main_f.pic_bmp.Modified:=true;
  main_f.pb_update(true);
end;

procedure stream_save(strm:tstream);
var
  r:integer;
  sprite:tsprite;
begin
  service_u.stream_write(strm,'<sprite_list>');
  service_u.stream_write(strm,'<count>'+inttostr(count)+'</count>');
  service_u.stream_write(strm,'</sprite_list>');

  for r:=0 to count-1 do begin
    sprite:=sprite_get(r);
    sprite.stream_save(strm);
  end;
end;

procedure stream_load(strm:tstream);
var
  sprite_list:string;
  max:integer;
  r:integer;
  sprite:tsprite;
begin
  sprite_list:=service_u.stream_tag_section_read(strm,'sprite_list');
  max:=service_u.string_tag_int(sprite_list,'count',0);

  for r:=0 to max-1 do begin
    sprite:=tsprite.create;
    sprite.stream_load(strm);
    add(sprite);
  end;
end;


function count:integer;
begin
  result:=main_f.sprite_lb.Items.count;
end;

procedure delete(inx:integer);
var
  sprite:tsprite;
begin
  if not inx_check(inx) then exit;
  sprite:=sprite_get(inx);
  sprite.Free;
  main_f.sprite_lb.Items.Delete(inx);
  main_f.pic_bmp.Modified:=true;
end;

procedure delete_active;
var
  sprite:tsprite;
begin
  sprite:=sprite_active;if sprite=nil then exit;
  if application.MessageBox('Are you shure?','Kill the sprite',mb_yesno)=id_no then exit;

  delete(main_f.sprite_lb.itemindex);
  main_f.pb_update(true);
end;

procedure clear;
var
  r:integer;
begin
  for r:=count-1 downto 0 do begin
    delete(r);
  end;
end;

function inx_check(inx:integer):bool;
begin
  result:=(inx>=0)and(inx<count);
end;

procedure clipboard_copy;
var
  sprite:tsprite;
  mstrm:tmemorystream;
begin
  if main_f.pctrl.activepage=main_f.selection_ts then exit;
  sprite:=sprite_active;if sprite=nil then exit;

  screen.cursor:=crhourglass;
  mstrm:=tmemorystream.Create;
  try
    sprite.stream_save(mstrm);
    service_u.stream_clipboard_copy(main_f.spp_clipboard_format,mstrm);
  finally
    mstrm.free;
    screen.cursor:=crdefault;
  end;
end;

procedure clipboard_paste;
var
  sprite:tsprite;
  mstrm:tmemorystream;
begin
  if main_f.pctrl.activepage=main_f.selection_ts then exit;

  screen.cursor:=crhourglass;
  sprite:=tsprite.create;
  mstrm:=tmemorystream.Create;
  try
    if not service_u.stream_clipboard_paste(main_f.spp_clipboard_format,mstrm) then begin
      sprite.Free;
      exit;
    end;
    sprite.stream_load(mstrm);
    main_f.sprite_lb.ItemIndex:=add(sprite);
    main_f.pb_update(true);
  finally
    mstrm.free;
    screen.cursor:=crdefault;
  end;
end;

end.

